# -*- coding: utf-8 -*- 
# @Time    : 04/13/2024 
# @Author  : nzooherd 
# @File    : __init__.py 
# @Software: Visual Studio Code


import abc
import json
import base64
from typing import Dict, Optional, Type
from telethon._impl.client.types import buttons as btns

from sqlalchemy import null


class TagMeta(Type):
    pass


class Tag(metaclass=TagMeta):

    def __init__(self, value: str = None) -> None:
        self._value = value
        self._tags = []
    
    def value(self, val: str):
        self._value = val
        return self

    def add(self, tag):
        self._tags.append(tag)
        return self
    
    def __str__(self) -> str:
        self_str = f"<{self.__tag__}>{"" if not self._value else self._value}%s</{self.__tag__}>"
        return self_str %  "".join([str(tag) for tag in self._tags])


class ATag(Tag):
    __tag__ = "a"

    def __init__(self, value=None, href=None) -> None:
        super().__init__(value)
        self.href = href

    def __str__(self) -> str:
        self_str = f"<a href={self.href}>{"" if not self._value else self._value}%s</a>"
        return self_str %  "".join([str(tag) for tag in self._tags])

class StrongTag(Tag):
    __tag__ = "strong"
    pass

class PTag(Tag):
    __tag__ = 'p'

class BTag(Tag):

    __tag__ = 'b'
    pass

class ITag(Tag):

    __tag__ = 'i'
    pass

class Details(Tag):

    __tag__ = 'details'
    pass



class BaseTemplate(metaclass=abc.ABCMeta):

    def __init__(self) -> None:
        self.head = """
**五星精选, 尽在无忧洞。**
🫦 🍑 🍒 💦 👅 🍆 🌶️ 🌭 👉 👌 ♋ 
"""
        self.buttons = []

    @abc.abstractmethod
    def template(self) -> Dict[str, str]:
        """
        返回给 telethon 发送的消息模板
        """
        pass

    def content(self) -> str:
        """
        内部消息记录的内容
        """
        return self.template().get('html')

    def button(self) -> str:
        """
        内部消息记录的按钮
        """
        json_data = []

        def btn2json(btn):
            return {"text": btn.text, "data": base64.b64encode(getattr(btn, "data", bytes(1))).decode("utf-8")}

        for btn in self.buttons:
            if isinstance(btn, list):
                json_data.append([btn2json(b) for b in btn])
            else:
                json_data.append(btn2json(btn))
        return json.dumps(json_data)

    
